#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# $Id: test-canvas-lines.tcl,v 1.6 2003/11/02 11:02:57 baum Exp $

# ensure that "." is the decimal point
set ::env(LC_NUMERIC) "C"

# an example with canvas lines
set auto_path [linsert $auto_path 0 \
      [file join [file dirname [info script]] ../../src]]
package require GnoclCanvas

set pi2 [expr {-2.*atan(1)}]

set canvas [gnocl::canvas -antialiased 1 -background white]
      # -scrollRegion {0 0 500 500} -scrollbar never]

$canvas create text -coords "1 1" -anchor NW -text \
      "Use left, middle and right mouse buttons to\
      move, rotate and scale items"

# $canvas create line -coords "-10 -10 10 10" -width 2
# $canvas create line -coords "-10 10 10 -10" -width 2

# capStyle
set x 200
set y 380
foreach cap {"notLast" "butt" "round" "projecting"} {
   $canvas create line -coords [list $x $y $x [expr {$y+100}]] -width 10 \
         -fill blue -capStyle $cap
   set tx [expr {$x-15}]
   set ty [expr {$y+50}]
   set txt [$canvas create text -coords [list $tx $ty] -text "capStyle $cap"]
   $canvas rotate $txt [list $tx $ty $pi2]
   incr x 50
}

# joinStyle
set x 250
set y 30
foreach joinStyle { "miter" "round" "bevel" } {
   $canvas create line -coords \
         [list  $x $y [expr {$x+130}] [expr {$y+20}] $x [expr {$y+40}] ] \
         -width 10 -fill blue -joinStyle $joinStyle
   set tx [expr {$x+120}]
   set ty $y
   set txt [$canvas create text -coords [list $tx $ty] \
         -text "joinStyle $joinStyle"]
   $canvas rotate $txt [list $tx $ty [expr {atan2(20,130)}]]
   incr y 50
}

# arrow side
set x 30
set y 270
foreach arrow {"none" "first" "last" "both"} {
   $canvas create line -coords [list $x $y $x [expr {$y+100}]] -width 5 \
         -fill red -arrow $arrow -arrowShape {5 10 5}
   set tx [expr {$x-15}]
   set ty [expr {$y+50}]
   set txt [$canvas create text -coords [list $tx $ty] -text "arrow $arrow"]
   $canvas rotate $txt [list $tx $ty $pi2]
   incr x 50 
}

# arrow form
set x 100
set y 70
foreach shape {{10 10 10} {20 10 10} {10 20 10} {10 10 20} } {
   $canvas create line -coords [list $x $y [expr {$x+100}] $y] -width 5 \
         -fill red -arrow both -arrowShape $shape
   $canvas create text -coords [list [expr {$x+50}] [expr {$y-20}]] \
         -text "arrowShape $shape"
   incr y 40
}

### #smooth # not yet implemented in gnome-canvas-line.c
### set x 290
### set y 220
### $canvas create text -coords [list [expr {$x+80}] $y] -anchor S \
###       -fill red -text "smooth does not work in GTK+!"
### foreach smooth {1 0} {
###    set x1 [expr {$x+100}]
###    set dx 20
###    $canvas create line -coords \
###          [list  $x $y $x1 [expr {$y+$dx/2}] \
###                $x [expr {$y+$dx}] $x1 [expr {$y+5*$dx/2}] \
###                $x [expr {$y+5*$dx}]] \
###          -smooth $smooth -splineSteps 100 -fill darkgreen 
###    incr x 60
### }

set buttonPress -1
set lastX 0
set lastY 0
set xCenter 0
set yCenter 0

proc moveItem { canv item x y } {
   global lastX lastY
   # puts [format "move: %.2f %.2f" [expr {$x-$lastX}] [expr {$y-$lastY}]]
   $canv move $item [list [expr {$x-$lastX}] [expr {$y-$lastY}]]
   set lastX $x
   set lastY $y
}
proc rotateItem { canv item x y } {
   global lastX lastY xCenter yCenter
   set a1 [expr {atan2($y-$yCenter,$x-$xCenter)}]
   set a2 [expr {atan2($lastY-$yCenter,$lastX-$xCenter)}]
   $canv rotate $item [list $xCenter $yCenter [expr {$a1-$a2}]]
   set lastX $x
   set lastY $y
}

proc scaleItem { canv item x y } {
   global lastX lastY xCenter yCenter
   set a1 [expr {pow($y-$yCenter,2)+pow($x-$xCenter,2)}]
   set a2 [expr {pow($lastY-$yCenter,2)+pow($lastX-$xCenter,2)}]
   if { $a1 > 1.e-5 && $a2 > 1.e-5 } {
      $canv scale $item [list $xCenter $yCenter [expr {sqrt($a1/$a2)}]]
      set lastX $x
      set lastY $y
   }
}

proc doOnButtonPress { canv item x y but } {
   global buttonPress lastX lastY xCenter yCenter
   if { $buttonPress < 0 } {
      foreach {x0 y0 x1 y1} [$canv getBounds $item] break
      set xCenter [expr {($x1+$x0)/2.}]
      set yCenter [expr {($y1+$y0)/2.}]
      # puts "$x0 $y0 $x1 $y1 $xCenter $yCenter"

      set buttonPress $but
      set lastX $x
      set lastY $y
      switch $but {
         1 { set script moveItem }
         2 { set script rotateItem }
         3 { set script scaleItem }
         default { return }
      }
      $canv itemConfigure $item -onMotion "$script %w %i %x %y"
   }
}

proc doOnButtonRelease { canv item but } {
   global buttonPress
   if { $buttonPress == $but } {
      $canv itemConfigure $item -onMotion ""
      set buttonPress -1
   }
}

$canvas itemConfigure all -onButtonPress "doOnButtonPress %w %i %x %y %b" \
      -onButtonRelease "doOnButtonRelease %w %i %b"

set win [gnocl::window -title "Canvas Line Test" -child $canvas \
      -defaultWidth 500 -defaultHeight 500 -onDestroy exit]

gnocl::mainLoop

