#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# $Id: test-canvas-richText.tcl,v 1.6 2003/11/02 11:02:57 baum Exp $

# ensure that "." is the decimal point
set ::env(LC_NUMERIC) "C"

set auto_path [linsert $auto_path 0 \
      [file join [file dirname [info script]] ../../src]]
package require GnoclCanvas

# in GTK+ 2.0.6 richText is only implemented for non-antialiased canvas
set right [gnocl::box -orientation vertical -homogeneous 0]
set left [gnocl::box -orientation vertical -homogeneous 1]
set mainBox [gnocl::box -orientation horizontal \
      -children $left -expand 1 -fill 1 -homogeneous 0]
$mainBox add $right -expand 0

set canvas [gnocl::canvas -background white -antialiased 0 \
      -onKeyPress {puts [list KeyPress: %w %k %K %a]} \
      -onButtonPress "%w configure -hasFocus 1"]

set text [gnocl::text]
$left add [list $canvas $text] -fill 1 -expand 1
$canvas create ellipse -coords {150 80 2} -fill blue
$canvas create richText -coords {150 80} -height 70 -tags text -anchor NW \
      -text "Hello World!\nThis is the second line.\nAnd this the third"
$canvas create richText -coords {10 170} -width 200 -tags tagText 
# GTK+ 2.0.6 does not allow to rotate richText
# $canvas rotate text "100 100 1.3"

$text tag create tag1 -foreground blue
$text tag create tag2
$canvas itemCommand tagText tag create tag1 -foreground blue
$canvas itemCommand tagText tag create tag2

$text insert end "This text has no tag\n"
$text insert end "This text has tag #1.\n" -tags tag1
$text insert end "This text has tag #2.\n" -tags tag2
$text insert end "NoTag" 
$text insert end "Tag#1"  -tags tag1
$text insert end "NoTag"
$text insert end "Tag#2"  -tags tag2
$text insert end "Tag#1"  -tags tag1

$canvas itemCommand tagText insert end "This text has no tag\n"
$canvas itemCommand tagText insert end "This text has tag #1\n" -tags tag1
$canvas itemCommand tagText insert end "This text has tag #2\n" -tags tag2
$canvas itemCommand tagText insert end "NoTag"
$canvas itemCommand tagText insert end "Tag#1" -tags tag1
$canvas itemCommand tagText insert end "NoTag"
$canvas itemCommand tagText insert end "Tag#2" -tags tag2
$canvas itemCommand tagText insert end "Tag#1" -tags tag1

proc setTags { tag val } {
   eval $::text tag configure $tag -$val
   eval $::canvas itemCommand tagText tag configure $tag -$val
}

set items {"fontStyle oblique" "fontWeight bold" "fontFamily courier" \
      "fontFamily helvetica" "foreground blue" "foreground red" \
      "background blue" "background white" "underline single" "underline none" } 
$right add [gnocl::label -text "tag1:"] -expand 0
$right add [gnocl::optionMenu -items $items \
      -onChanged "setTags tag1 %v"]
$right add [gnocl::label -text "tag2:"] -expand 0
$right add [gnocl::optionMenu -items $items \
      -onChanged "setTags tag2 %v"]

$right add [gnocl::checkButton -text "editable" -active 1 \
      -onToggled "$canvas itemConfigure text -editable %v"] -expand 0
$right add [gnocl::checkButton -text "visible" -active 1 \
      -onToggled "$canvas itemConfigure text -visible %v"] -expand 0
$right add [gnocl::checkButton -text "cursorVisible" -active 1 \
      -onToggled "$canvas itemConfigure text -cursorVisible %v"] -expand 0
$right add [gnocl::checkButton -text "cursorBlink" -active 1 \
      -onToggled "$canvas itemConfigure text -cursorBlink %v"] -expand 0
$right add [gnocl::checkButton -text "growHeight" -active 0 \
      -onToggled "$canvas itemConfigure text -growHeight %v"] -expand 0

$right add [gnocl::label -text "wrapMode:"] -expand 0
$right add [gnocl::optionMenu -items {word char none} \
      -onChanged "$canvas itemConfigure text -wrapMode %v"]
$right add [gnocl::label -text "justify:"] -expand 0
$right add [gnocl::optionMenu -items {left right center} \
      -onChanged "$canvas itemConfigure text -justify %v"]
$right add [gnocl::label -text "anchor:"] -expand 0
$right add [gnocl::optionMenu -items {center N NW NE S SW SE W E} \
      -onChanged "$canvas itemConfigure text -anchor %v" -value NW]

$right add [gnocl::label -text "pixelsAboveLines:"] -expand 0
$right add [gnocl::spinButton -digits 0 -upper 30 \
      -onValueChanged "$canvas itemConfigure text -pixelsAboveLines %v"] \
      -expand 0
$right add [gnocl::label -text "pixelsBelowLines:"] -expand 0
$right add [gnocl::spinButton -digits 0 -upper 30 \
      -onValueChanged "$canvas itemConfigure text -pixelsBelowLines %v"] \
      -expand 0
$right add [gnocl::label -text "pixelsInsideWrap:"] -expand 0
$right add [gnocl::spinButton -digits 0 -upper 30 \
      -onValueChanged "$canvas itemConfigure text -pixelsInsideWrap %v"] \
      -expand 0
$right add [gnocl::label -text "leftMargin:"] -expand 0
$right add [gnocl::spinButton -digits 0 -upper 30 \
      -onValueChanged "$canvas itemConfigure text -leftMargin %v"] \
      -expand 0
$right add [gnocl::label -text "rightMargin:"] -expand 0
$right add [gnocl::spinButton -digits 0 -upper 30 \
      -onValueChanged "$canvas itemConfigure text -rightMargin %v"] \
      -expand 0
$right add [gnocl::label -text "indent:"] -expand 0
$right add [gnocl::spinButton -digits 0 -upper 30 \
      -onValueChanged "$canvas itemConfigure text -indent %v"] \
      -expand 0

proc bgerror { err } {
   puts "in bgerror: $err"
   puts $::errorInfo
   # puts "errorCode: $::errorCode"
}

set win [gnocl::window -child $mainBox -onDestroy exit \
         -defaultHeight 500 -defaultWidth 500]

gnocl::mainLoop

