/*
 * $Id: canvas.c,v 1.4 2004/01/12 20:27:56 baum Exp $
 *
 * This file implements the canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2003-10: added onKey{Press,Release}
   2002-11: updates for gnome-2.0
            switched from GnoclWidgetOptions to GnoclOption
            added support for tags
        11: added scroll window
        05: added rectangle, line, text
   2001-03: Begin of developement
            first item is ellipse
 */

#include "canvas.h"
#include <string.h>
#include <math.h>
#include <assert.h>

#define GET_INFO(items,idx) \
      ((Gnocl_CanvasItemInfo *)g_ptr_array_index( items, idx ))

enum { Affine, Move, Scale, Rotate };

static GnoclOption canvasOptions[] = 
{
   { "-antialiased", GNOCL_BOOL, NULL },
   { "-centerScroll", GNOCL_BOOL, NULL },
   { "-scrollRegion", GNOCL_LIST, NULL },
   { "-width", GNOCL_INT, "width" },
   { "-height", GNOCL_INT, "height" },
   { "-background", GNOCL_OBJ, "normal", gnoclOptGdkColorBg },
   { "-onButtonPress", GNOCL_OBJ, "P", gnoclOptOnButton },
   { "-onButtonRelease", GNOCL_OBJ, "R", gnoclOptOnButton },
   { "-onMotion", GNOCL_OBJ, "", gnoclOptOnMotion },
   { "-onKeyPress", GNOCL_OBJ, "", gnoclOptOnKeyPress },
   { "-onKeyRelease", GNOCL_OBJ, "", gnoclOptOnKeyRelease },
   { "-name", GNOCL_STRING, "name" },
   { "-visible", GNOCL_BOOL, "visible" }, 
   { "-hasFocus", GNOCL_BOOL, "has-focus" },
   { NULL }
};

static const int antialiasedIdx  = 0;
static const int centerScrollIdx = 1;
static const int scrollRegionIdx = 2;

/* TODO?: { "-scrollbar" } */

typedef struct CanvasParams_
{
   char         *name;
   Tcl_Interp   *interp;
   GnomeCanvas  *canvas;
   GHashTable   *tags;
} CanvasParams;

static Gnocl_CanvasItemInfo *currentConfigItem = NULL;


static void itemEventFunc( GtkWidget *widget, GdkEvent *event, 
      gpointer data)
{
   Gnocl_CanvasItemInfo *info = (Gnocl_CanvasItemInfo *)data;
   int ns;

   switch( event->type )
   {
      case GDK_MOTION_NOTIFY:  ns = GNOCL_CANVAS_ON_MOTION; break;
      case GDK_ENTER_NOTIFY:   ns = GNOCL_CANVAS_ON_ENTER; break;
      case GDK_LEAVE_NOTIFY:   ns = GNOCL_CANVAS_ON_LEAVE; break;
      case GDK_BUTTON_PRESS:   
      case GDK_2BUTTON_PRESS:   
      case GDK_3BUTTON_PRESS:   
                               ns = GNOCL_CANVAS_ON_BUTTON_PRESS; break;
      case GDK_BUTTON_RELEASE: ns = GNOCL_CANVAS_ON_BUTTON_RELEASE; break;
      default:  return;
   }

   if( info->scripts[ns] == NULL )
      return;

   switch( event->type )
   {
      case GDK_MOTION_NOTIFY:  
               {
                  GnoclPercSubst ps[] = {
                     { 'w', GNOCL_STRING },
                     { 'i', GNOCL_INT },
                     { 'x', GNOCL_DOUBLE },
                     { 'y', GNOCL_DOUBLE },
                     { 's', GNOCL_INT },
                     { 0 }
                  };
                  ps[0].val.str = info->canvasParams->name;
                  ps[1].val.i = info->id;
                  ps[2].val.d = event->motion.x;
                  ps[3].val.d = event->motion.y;
                  ps[4].val.i = event->motion.state;  
                  gnoclPercentSubstAndEval( info->canvasParams->interp, 
                        ps, info->scripts[ns], 1 );
               }
               return;
      case GDK_ENTER_NOTIFY:  
      case GDK_LEAVE_NOTIFY:
               {
                  GnoclPercSubst ps[] = {
                     { 'w', GNOCL_STRING },
                     { 'i', GNOCL_INT },
                     { 'x', GNOCL_DOUBLE },
                     { 'y', GNOCL_DOUBLE },
                     { 's', GNOCL_INT },
                     { 0 }
                  };
                  ps[0].val.str = info->canvasParams->name;
                  ps[1].val.i = info->id;
                  ps[2].val.d = event->crossing.x;
                  ps[3].val.d = event->crossing.y;
                  ps[4].val.i = event->crossing.state;  
                  gnoclPercentSubstAndEval( info->canvasParams->interp, 
                        ps, info->scripts[ns], 1 );
               }
               return;

       case GDK_BUTTON_PRESS:
       case GDK_2BUTTON_PRESS:
       case GDK_3BUTTON_PRESS:
       case GDK_BUTTON_RELEASE:
               {
                  GnoclPercSubst ps[] = {
                     { 'w', GNOCL_STRING },
                     { 'i', GNOCL_INT },
                     { 'x', GNOCL_DOUBLE },
                     { 'y', GNOCL_DOUBLE },
                     { 's', GNOCL_INT },
                     { 'b', GNOCL_INT },
                     { 't', GNOCL_STRING },
                     { 0 }
                  };
                  ps[0].val.str = info->canvasParams->name;
                  ps[1].val.i = info->id;
                  ps[2].val.d = event->button.x;
                  ps[3].val.d = event->button.y;
                  ps[4].val.i = event->button.state;  
                  ps[5].val.i = event->button.button;  
                  switch( event->type )
                  {
                     case GDK_BUTTON_PRESS:  ps[6].val.str = "buttonPress"; 
                                             break;
                     case GDK_2BUTTON_PRESS: ps[6].val.str = "button2Press"; 
                                             break;
                     case GDK_3BUTTON_PRESS: ps[6].val.str = "button3Press"; 
                                             break;
                     case GDK_BUTTON_RELEASE: ps[6].val.str = "buttonRelease"; 
                                              break;
                     default:  assert( 0 ); break;
                  }
                  gnoclPercentSubstAndEval( info->canvasParams->interp, 
                        ps, info->scripts[ns], 1 );
               }
               return;
      default:  break;
   }

}

static int connectItemCommand( Tcl_Interp *interp, GObject *obj, 
      int sig, const char *cmd )
{
   /* currentConfigItem is an ugly hack to move the information
      which item is configure in this function. It should be
      nevertheless be safe since connectItemCommand cannot be
      called recursively.
   */
   Gnocl_CanvasItemInfo *info = currentConfigItem;
   int isConnected = 0;
   int k;

   assert( info->item == GNOME_CANVAS_ITEM( obj ) );

   for( k = 0; k < GNOCL_CANVAS_ON_COUNT; ++k )
      if( info->scripts[k] != NULL )
      {
         isConnected = 1;
         break;
      }

   g_free( info->scripts[sig] );

   if( cmd && *cmd )
   {
      info->scripts[sig] = g_strdup( cmd );
      if( isConnected == 0 )
         g_signal_connect_data( obj, "event", (GCallback)itemEventFunc,
               info, NULL, (GConnectFlags)0 );
   }
   else
   {
      info->scripts[sig] = NULL;
      
      for( k = 0; k < GNOCL_CANVAS_ON_COUNT; ++k )
      {
         if( info->scripts[k] != NULL )
            break;
      }
      if( k == GNOCL_CANVAS_ON_COUNT )
      {
         g_signal_handlers_disconnect_matched( obj, G_SIGNAL_MATCH_FUNC,
                0, 0, NULL, (gpointer *)itemEventFunc, NULL );
      }
   }

   return TCL_OK;
}

int gnoclItemOptOnFunc( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret )
{
   int sn;
   switch( opt->optName[3] )
   {
      case 'M':   sn = GNOCL_CANVAS_ON_MOTION;  break;
      case 'E':   sn = GNOCL_CANVAS_ON_ENTER;   break;
      case 'L':   sn = GNOCL_CANVAS_ON_LEAVE;   break;
      case 'B':   if( opt->optName[9] == 'P' )
                     sn = GNOCL_CANVAS_ON_BUTTON_PRESS;
                  else
                     sn = GNOCL_CANVAS_ON_BUTTON_RELEASE;
   }

#ifndef NDEBUG
   {
      const char *str;
      switch( sn )
      {
         case GNOCL_CANVAS_ON_MOTION: str = "-onMotion"; break;
         case GNOCL_CANVAS_ON_ENTER: str = "-onEnter"; break;
         case GNOCL_CANVAS_ON_LEAVE: str = "-onLeave"; break;
         case GNOCL_CANVAS_ON_BUTTON_PRESS: str = "-onButtonPress"; break;
         case GNOCL_CANVAS_ON_BUTTON_RELEASE: str = "-onButtonRelease"; break;
      }
      assert( strcmp( opt->optName, str ) == 0 );
   }
#endif

   return connectItemCommand( interp, obj, sn, Tcl_GetString( opt->val.obj ) );
}

static GPtrArray *getCanvasItemsFromTagOrId( CanvasParams *para, char *tag )
{
   /* TODO: allow calculation with &&, ||, !, ^ and () */

   /* !!!!!!!!!!!!!!!!!!

      if this is changed to create a GPtrArray and not to return a
      direct reference to the hash table, itemDelete must be changed!

      !!!!!!!!!!!!!!!!!!
   */
   GPtrArray *p = g_hash_table_lookup( para->tags, tag );
   return p;
}

static int canvasAddTag( Tcl_Interp *interp, CanvasParams *para, 
      Gnocl_CanvasItemInfo *item, const char *tag )
{
   GPtrArray *tags = g_hash_table_lookup( para->tags, tag );
   if( tags == NULL )
   {
      /* TODO? use also hash? */
      tags = g_ptr_array_new( );
      g_hash_table_insert( para->tags, g_strdup( tag ), tags );
      /* printf( "new array %p for tag %s\n", tags, tag ); */
   }
   g_ptr_array_add( tags, item );
   g_ptr_array_add( item->tags, g_strdup( tag ) );
   /* printf( "adding %p to tag %s\n", item, tag ); */

   return TCL_OK;
}

static void destroyItemFunc( GnomeCanvasItem *item, gpointer data )
{
   Gnocl_CanvasItemInfo *info = (Gnocl_CanvasItemInfo *)data;
   guint k;

   assert( info->item == item );
   for( k = 0; k < info->tags->len; ++k )
   {
      char *tag = (char *)g_ptr_array_index( info->tags, k );
      GPtrArray *tags = g_hash_table_lookup( info->canvasParams->tags, tag );
      int rm = g_ptr_array_remove_fast( tags, info );
      /* printf( "removed item %p from %s\n", info, tag ); */
      assert( rm );
      if( tags->len == 0 )
      {
         rm = g_hash_table_remove( info->canvasParams->tags, tag );
         /* printf( "removed array %s\n", tag ); */
         assert( rm );
      }
      g_free( tag );
   }
   g_ptr_array_free( info->tags, 1 );

   for( k = 0; k < GNOCL_CANVAS_ON_COUNT; ++k )
      g_free( info->scripts[k] );
}

static void destroyFunc( GtkWidget *widget, gpointer data )
{
   CanvasParams *p = (CanvasParams *)data;
   GPtrArray    *items;

   gnoclForgetWidgetFromName( p->name );
   Tcl_DeleteCommand( p->interp, p->name );

   g_free( p->name );
   /*  TODO: the canvas receives the destroy signal before its items. But
             the items use the hash table on destruction.
             Shouldn't children receive the signal before their parents?
             Hmm, GTK widgets do it the same way.  
             gtk_container_foreach( GTK_CONTAINER( widget ), 
                   (GtkCallback)destroyItemFunc, NULL);  
             doesn't help either

   */
   items = getCanvasItemsFromTagOrId( p, "all" );
   if( items != NULL )
   {
      /* this works since items is a reference to the original array,
         so if we delete one item, items->len is decreased at once by one */
      while( items->len )
      {
         Gnocl_CanvasItemInfo *info = GET_INFO( items, 0 );
         gtk_object_destroy( GTK_OBJECT( info->item ) );
      }
   }
   g_hash_table_destroy( p->tags ); 
   g_free( p );
}

int gnoclCanvasHandleTags( Tcl_Interp *interp, GnoclOption *opt, 
      Gnocl_CanvasItemInfo *info )
{
   if( opt->status == GNOCL_STATUS_CHANGED )
   {
      int no, k;
      if( Tcl_ListObjLength( interp, opt->val.obj, &no ) != TCL_OK  )
         return TCL_ERROR;
      for( k = 0; k < no; ++k )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, opt->val.obj, k, &tp ) != TCL_OK )
            return TCL_ERROR;
         /* TODO? tag must not start with a digit and must not contain 
                  any of "&|^!()" */
         canvasAddTag( interp, info->canvasParams, info, Tcl_GetString( tp ) );
      }
   }

   return TCL_OK;
}

static Gnocl_CanvasItemInfo *getInfoFromCanvasItem( CanvasParams *param, 
      GnomeCanvasItem *item )
{
   if( item != NULL )
   {
      GPtrArray *items = getCanvasItemsFromTagOrId( param, "all" );
      guint n;
      for( n = 0; n < items->len; ++n )
      {
         Gnocl_CanvasItemInfo *info = GET_INFO( items, n );
         if( info->item == item )
            return info;
      }
   }
   return NULL;
}

static int configure( Tcl_Interp *interp, CanvasParams *para,
      GnoclOption options[] )
{
   if( options[scrollRegionIdx].status == GNOCL_STATUS_CHANGED )
   {
      Tcl_Obj *obj = options[scrollRegionIdx].val.obj;
      int     k, no;
      double  val[4];   /* x, y, w, h */

      if( Tcl_ListObjLength( interp, obj, &no ) != TCL_OK 
         || no != 4 )
      {
         Tcl_SetResult( interp, 
               "scrollRegion must be proper list with four members", 
               TCL_STATIC );
         return TCL_ERROR;
      }
      for( k = 0; k < no; ++k )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, obj, k, &tp ) != TCL_OK )
            return TCL_ERROR;
         if( Tcl_GetDoubleFromObj( interp, tp, &val[k] ) )
            return TCL_ERROR;
      }
      gnome_canvas_set_scroll_region( para->canvas, 
            val[0], val[1], val[0] + val[2], val[1] + val[3] ); 
   }

   if( options[centerScrollIdx].status == GNOCL_STATUS_CHANGED )
   {
      gnome_canvas_set_center_scroll_region( para->canvas, 
            options[centerScrollIdx].val.b );
   }

#if 0
   if( popt->scrollbar.changed )
   {
      GtkPolicyType hor, vert;
      if( gnoclGetScrollbarPolicy( interp, popt->scrollbar.val, 
            &hor, &vert ) != TCL_OK )
         return TCL_ERROR;

      gtk_scrolled_window_set_policy( para->scrollWin, hor, vert );
   }
#endif

   return TCL_OK;
}

static int windowToWorld( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], CanvasParams *params, int reverse )
{
   Tcl_Obj *resList;
   int     noCoords, n;
   if( objc != 3 )
   {
      Tcl_WrongNumArgs( interp, 2, objv, 
            /* canvas windowToWorld */
            "list-of-coordinates ?option val ...?" );
      return TCL_ERROR;
   }
   /* TODO  
         -only [xy]:          only x, y coordinates
         -pairs [true|false]: list of coordinate pairs (lists)
   */
   if( Tcl_ListObjLength( interp, objv[2], &noCoords ) != TCL_OK
         || ( noCoords % 2 ) )
   {
      Tcl_SetResult( interp, 
            "size of list-of-coordinates must be even", 
            TCL_STATIC );
      return TCL_ERROR;
   }
   resList = Tcl_NewListObj( 0, NULL );
   for( n = 0; n < noCoords; n += 2 )
   {
      Tcl_Obj *tp;
      double xw, yw, x, y;
      int ret = Tcl_ListObjIndex( interp, objv[2], n, &tp );
      if( ret == TCL_OK )
         ret = Tcl_GetDoubleFromObj( interp, tp, &xw );
      if( ret == TCL_OK )
         ret = Tcl_ListObjIndex( interp, objv[2], n + 1, &tp );
      if( ret == TCL_OK )
         ret = Tcl_GetDoubleFromObj( interp, tp, &yw );

      if( ret != TCL_OK )
      {
         Tcl_DecrRefCount( resList );  /* FIXME: is this correct? */
         return TCL_ERROR;
      }
      if( reverse )
         gnome_canvas_world_to_window( params->canvas, xw, yw, &x, &y );
      else
         gnome_canvas_window_to_world( params->canvas, xw, yw, &x, &y );
      Tcl_ListObjAppendElement( interp, resList, Tcl_NewDoubleObj( x ) );
      Tcl_ListObjAppendElement( interp, resList, Tcl_NewDoubleObj( y ) );
   }
   Tcl_SetObjResult( interp, resList );

   return TCL_OK;
}

static int findItemAt( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], CanvasParams *params )
{
   double   x, y;
   GnomeCanvasItem *item;
   Gnocl_CanvasItemInfo *info;

   if( objc != 4 )
   {
      Tcl_WrongNumArgs( interp, 2, objv, /* canvas findItemAt */ "x y" );
      return TCL_ERROR;
   }

   if( Tcl_GetDoubleFromObj( interp, objv[2], &x ) != TCL_OK )
      return TCL_ERROR;
   if( Tcl_GetDoubleFromObj( interp, objv[3], &y ) != TCL_OK )
      return TCL_ERROR;
   
   item = gnome_canvas_get_item_at( params->canvas, x, y );
   /* printf( "item: %p %f %f\n", item, x, y ); */
   info = getInfoFromCanvasItem( params, item );
   if( info )
      Tcl_SetObjResult( interp, Tcl_NewIntObj( info->id ) );

   return TCL_OK;
}

static int itemBounds( Tcl_Interp *interp, int objc, Tcl_Obj * const objv[], 
      CanvasParams *param, GPtrArray *items )
{
   Gnocl_CanvasItemInfo *info;

   Tcl_Obj *resList;
   double  xMin, yMin, xMax, yMax;
   guint   k;

   if( objc != 3 )
   {
      Tcl_WrongNumArgs( interp, 2, objv, NULL );
      return TCL_ERROR;
   }

   assert( items->len >= 1 );

   info = GET_INFO( items, 0 );
   gnome_canvas_item_get_bounds( info->item, &xMin, &yMin, &xMax, &yMax );

   for( k = 1; k < items->len; ++k )
   {
      double x1, y1, x2, y2;
      info = GET_INFO( items, k );
      gnome_canvas_item_get_bounds( info->item, &x1, &y1, &x2, &y2 );
      if( x1 < xMin )
         xMin = x1;
      if( y1 < yMin )
         yMin = y1;
      if( x2 > xMax )
         xMax = x2;
      if( y2 > yMax )
         yMax = y2;
   }

   resList = Tcl_NewListObj( 0, NULL );
   Tcl_ListObjAppendElement( interp, resList, Tcl_NewDoubleObj( xMin ) );
   Tcl_ListObjAppendElement( interp, resList, Tcl_NewDoubleObj( yMin ) );
   Tcl_ListObjAppendElement( interp, resList, Tcl_NewDoubleObj( xMax ) );
   Tcl_ListObjAppendElement( interp, resList, Tcl_NewDoubleObj( yMax ) );
   Tcl_SetObjResult( interp, resList );

   return TCL_OK;
}

static int itemCommand( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], CanvasParams *param, GPtrArray *items )
{
   /* canvas itemCommand tag-or-id command ?option val ...? */
   guint k;
   if( objc < 4 )
   {
      Tcl_WrongNumArgs( interp, 3, objv, "command ?option val ...?" );
      return TCL_ERROR;
   }

   for( k = 0; k < items->len; ++k )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, k );
      if( info->command == NULL )
      {
         Tcl_SetResult( interp, "Canvas item does not have any command.", 
            TCL_STATIC );
         return TCL_ERROR;
      }
      if( (*info->command)( interp, objc, objv, info ) != TCL_OK )
         return TCL_ERROR;
   }

   return TCL_OK;
}


static int itemConfigure( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], CanvasParams *param, GPtrArray *items )
{
   /* canvas itemConfigure tag-or-id ?option val ...? */
   guint k;
   int   ret = TCL_ERROR;

   for( k = 0; k < items->len; ++k )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, k );
      if( (*info->parseOptions)( interp, objc - 2, objv + 2, info ) != TCL_OK )
         goto cleanExit;
   }
   for( k = 0; k < items->len; ++k )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, k );
      currentConfigItem = info;
      if( (*info->setOptions)( interp, info ) != TCL_OK )
         goto cleanExit;
   }

   ret = TCL_OK;

cleanExit:
   for( k = 0; k < items->len; ++k )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, k );
      (*info->clearOptions)( interp, info );
   }

   return ret;
}

static int itemShow( Tcl_Interp *interp, int objc, Tcl_Obj * const objv[], 
      CanvasParams *param, GPtrArray *items )
{
   guint k;
   int   on = 1;

   if( objc > 4 )
   {
      Tcl_WrongNumArgs( interp, 3, objv, "?on?" );
      return TCL_ERROR;
   }
   if( objc == 4 )
   {
      if( Tcl_GetBooleanFromObj( interp, objv[3], &on ) != TCL_OK )
         return TCL_ERROR;
   }
      
   
   for( k = 0; k < items->len; ++k )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, k );
      if( on )
         gnome_canvas_item_show( info->item );
      else
         gnome_canvas_item_hide( info->item );
   }

   return TCL_OK;
}

static int itemDelete( Tcl_Interp *interp, int objc, Tcl_Obj * const objv[], 
      CanvasParams *param, GPtrArray *items )
{
   if( objc != 3 )
   {
      Tcl_WrongNumArgs( interp, 3, objv, NULL );
      return TCL_ERROR;
   }

   /* this works since items is a reference to the original array,
      so if we delete one item, items->len is decreased at once by one */
   while( items->len )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, 0 );
      gtk_object_destroy( GTK_OBJECT( info->item ) );
   }

   return TCL_OK;
}

static int canvasCreateItem( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], CanvasParams *param )
{
   const char *items[] = { "line", "rectangle", "ellipse", "bPath",
         "text", "richText", "widget",
         NULL };
   enum itemIdx { LineIdx, RectangleIdx, EllipseIdx,  BPathIdx,
         TextIdx, RichTextIdx, WidgetIdx };

   static int noItems = 0;

   Gnocl_CanvasItemInfo *info;
   GnoclItemCreateFunc *func;
   int  idx;
   int  k;
   char buffer[64];

   if( objc < 3 )
   {
      Tcl_WrongNumArgs( interp, 2, objv, 
            /* canvas create */ "type ?option val ...?" );
      return TCL_ERROR;
   }

   if( Tcl_GetIndexFromObj( interp, objv[2], items, "item type", 
         TCL_EXACT, &idx ) != TCL_OK )
      return TCL_ERROR;

   switch( idx )
   {
      case LineIdx:        func = gnoclCanvasCreateLine; break;
      case RectangleIdx:   func = gnoclCanvasCreateRectangle; break;
      case EllipseIdx:     func = gnoclCanvasCreateEllipse; break;
      case BPathIdx:       func = gnoclCanvasCreateBPath; break;
      case TextIdx:        func = gnoclCanvasCreateText; break;
      case RichTextIdx:    func = gnoclCanvasCreateRichText; break;
      case WidgetIdx:      func = gnoclCanvasCreateWidget; break;
      default:             assert( 0 );
                           return TCL_ERROR; /* should never happen */
   }

   info = (*func)( interp, objc, objv, gnome_canvas_root( param->canvas ) );
   if( info == NULL )
      return TCL_ERROR;

   info->id = ++noItems;
   info->tags = g_ptr_array_new( );
   info->canvasParams = param;
   for( k = 0; k < GNOCL_CANVAS_ON_COUNT; ++k )
      info->scripts[k] = NULL;

   sprintf( buffer, "%d", noItems );
   canvasAddTag( interp, param, info, buffer );
   canvasAddTag( interp, param, info, "all" );

   if( (*info->parseOptions)( interp, objc - 2, objv + 2, info ) != TCL_OK )
      return TCL_ERROR; /* TODO: free all */

   currentConfigItem = info;
   if( (*info->setOptions)( interp, info ) != TCL_OK )
      return TCL_ERROR; /* TODO: free all */

   (*info->clearOptions)( interp, info );
   
   g_signal_connect( G_OBJECT( info->item ), "destroy", 
         G_CALLBACK( destroyItemFunc ), info );

   /* gnome_canvas_update_now( param->canvas ); */
   Tcl_SetObjResult( interp, Tcl_NewIntObj( noItems ) );

   return TCL_OK;
}

static int affine( Tcl_Interp *interp, int objc, Tcl_Obj * const objv[], 
      CanvasParams *param, GPtrArray *items, int type )
{
   GnoclOption options[] = {
      { "-absolute", GNOCL_BOOL, NULL },
      { "-reverseOrder", GNOCL_BOOL, NULL },
      { NULL }
   };
   static const int absoluteIdx     = 0;
   static const int reverseOrderIdx = 1;

   guint m; 
   int   noCoords;
   int   absolute = 0; 
   int   reverse = 0;

   if( objc < 4  )
   {
      /* canvas (affine|move|scale|rotate) tag-or-id  coords ?options ? */
      Tcl_WrongNumArgs( interp, 3, objv, 
            "list-of-coordinates ?option val ...?" );
      return TCL_ERROR;
   }
   if( gnoclParseOptions( interp, objc - 3, objv + 3, options ) != TCL_OK )
      return TCL_ERROR;

   if( options[absoluteIdx].status == GNOCL_STATUS_CHANGED )
      absolute = options[absoluteIdx].val.b;
   if( options[reverseOrderIdx].status == GNOCL_STATUS_CHANGED )
      reverse = options[reverseOrderIdx].val.b;
   gnoclClearOptions( options );

   if( absolute && reverse )
   {
      Tcl_SetResult( interp, "Options \"-reverseOrder\" is only valid "
            "for relative transformations.", TCL_STATIC );
      return TCL_ERROR;
   }

   if( Tcl_ListObjLength( interp, objv[3], &noCoords ) != TCL_OK )
      return TCL_ERROR;

   for( m = 0; m < items->len; ++m )
   {
      Gnocl_CanvasItemInfo *info = GET_INFO( items, m );
      int k;
      double af[6];
      Tcl_Obj *tp;

      for( k = 0; k < 6 && k < noCoords; ++k )
      {
         if( Tcl_ListObjIndex( interp, objv[3], k, &tp ) != TCL_OK )
            return TCL_ERROR;
         if( Tcl_GetDoubleFromObj( interp, tp, &af[k] ) != TCL_OK )
            return TCL_ERROR;
      }

      /*
         x' = af[0] * x + af[2] * y + af[4];
         y' = af[1] * x + af[3] * y + af[5];
      */
      switch( type )
      {
         case Affine:
               if( noCoords != 6 )
               {
                  Tcl_SetResult( interp, 
                        "size of list-of-coordinates must be 6", TCL_STATIC );
                  return TCL_ERROR;
               }
               break;
         case Move:
               if( noCoords != 2 )
               {
                  Tcl_SetResult( interp, "size of list-of-coordinates must "
                        "be 2 (delta-x and delta-y)", TCL_STATIC );
                  return TCL_ERROR;
               }
               af[4] = af[0];
               af[5] = af[1];

               af[0] = 1.; af[2] = 0.;
               af[1] = 0.; af[3] = 1.;
               break;
         case Scale:
               if( noCoords != 3 && noCoords != 4 )
               {
                  Tcl_SetResult( interp, "size of list-of-coordinates must "
                        "be 3 or 4 (center, x-scale and y-scale)", 
                        TCL_STATIC );
                  return TCL_ERROR;
               }
               else
               {
                  double x = af[0];
                  double y = af[1];
                  double scalex = af[2];
                  double scaley = (noCoords == 4) ? af[3] : scalex;
                  af[0] = scalex; af[2] = .0;     af[4] = x * (1. - scalex);
                  af[1] = .0;     af[3] = scaley; af[5] = y * (1. - scaley);
               }
               break;
         case Rotate:
               /* there seems to be some problems with the 
                  canvas without antialiasing and rotating 
                  (bizarre drawings)
               */
               if( noCoords != 3 )
               {
                  Tcl_SetResult( interp, "size of list-of-coordinates must "
                        "be 3 (center and angle)", TCL_STATIC );
                  return TCL_ERROR;
               }
               else
               {
                  double x = af[0];
                  double y = af[1];
                  double w = af[2];
                  const double cw = cos( w );
                  const double sw = sin( w );

                  af[0] = cw; af[2] = -sw; af[4] = x - x * cw + y * sw;
                  af[1] = sw; af[3] =  cw; af[5] = y - x * sw - y * cw;
               }
               break;
      }

      /*
      printf( "\nitem: %p absolute: %d\n", info->item, absolute );
      printf( "before:\n" );
      {
         int k;
         double af[6];
         gnome_canvas_item_i2c_affine( info->item, af );
         for( k = 0; k < 6; k += 2 )
            printf( "%d: %f ", k, af[k] );
         printf( "\n" );
         for( k = 1; k < 6; k += 2 )
            printf( "%d: %f ", k, af[k] );
         printf( "\n" );
      }
      printf( "affine:\n" );
      for( k = 0; k < 6; k += 2 )
         printf( "%d: %f ", k, af[k] );
      printf( "\n" );
      for( k = 1; k < 6; k += 2 )
         printf( "%d: %f ", k, af[k] );
      printf( "\n" );
      */

      if( absolute )
         gnome_canvas_item_affine_absolute( info->item, af );
      else
      {
         /* Well, *I* think this is a bug in gnome: 
            affine_relative should not be the reverse of what one
            would expect. Or do I have wrong expectations? */
         if( reverse )
            gnome_canvas_item_affine_relative( info->item, af );
         else
         {
            double bf[6];
            gnome_canvas_item_i2c_affine( info->item, bf );
            gnome_canvas_item_affine_absolute( info->item, af );
            gnome_canvas_item_affine_relative( info->item, bf );
         }
      }
   }

   return TCL_OK;
}

static int canvasFunc( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{
   const char *cmds[] = { "delete", "configure", 
         "create", "itemDelete", "itemHide", "itemShow",
         "itemConfigure", "itemCommand", 
         "affine", "scale", "move", "rotate",
         "windowToWorld", "worldToWindow",
         "findItemAt", "getBounds",
         NULL };
   enum cmdIdx { DeleteIdx, ConfigureIdx, 
         CreateIdx, ItemDeleteIdx, ItemHideIdx, ItemShowIdx,
         ItemConfigureIdx, ItemCommandIdx, 
         AffineIdx, ScaleIdx, MoveIdx, RotateIdx,
         WindowToWorldIdx, WorldToWindowIdx,
         FindItemAtIdx, GetBoundsIdx };
   CanvasParams *para = (CanvasParams *)data;
   int idx;

   if( objc < 2 )
   {
      Tcl_WrongNumArgs( interp, 1, objv, "command" );
      return TCL_ERROR;
   }

   if( Tcl_GetIndexFromObj( interp, objv[1], cmds, "command", 
         TCL_EXACT, &idx ) != TCL_OK )
      return TCL_ERROR;

   switch( idx )
   {
      case DeleteIdx:
            return gnoclDelete( interp, GTK_WIDGET( para->canvas ), 
                  objc, objv );

      case ConfigureIdx:
            {
               int ret = TCL_ERROR;
               if( gnoclParseAndSetOptions( interp, objc - 1, objv + 1, 
                     canvasOptions, G_OBJECT( para->canvas ) ) == TCL_OK )
               {

                  if( canvasOptions[antialiasedIdx].status 
                        == GNOCL_STATUS_CHANGED )
                  {
                     Tcl_SetResult( interp, "antialiasing cannot be changed "
                           "after creation", TCL_STATIC );
                  }
                  else
                     ret = configure( interp, para, canvasOptions );
               }
               gnoclClearOptions( canvasOptions );
               return ret;
            }
            break;
      case CreateIdx:
            return canvasCreateItem( interp, objc, objv, para );
      case ItemDeleteIdx:
      case ItemHideIdx:
      case ItemShowIdx:
      case ItemConfigureIdx:
      case ItemCommandIdx:
      case AffineIdx:
      case ScaleIdx:
      case MoveIdx:
      case RotateIdx:
      case GetBoundsIdx:
            {
               GPtrArray *items;

               if( objc < 3 )
               {
                  Tcl_WrongNumArgs( interp, 2, objv, 
                        "tag-or-id ?option val ...?" );
                  return TCL_ERROR;
               }
               items = getCanvasItemsFromTagOrId( para, 
                     Tcl_GetString( objv[2] ) );
               if( items == NULL )
                  return TCL_OK;

               switch( idx )
               {
                  case ItemDeleteIdx:
                     return itemDelete( interp, objc, objv, para, items );
                  case ItemShowIdx:
                     return itemShow( interp, objc, objv, para, items );
                  case ItemConfigureIdx:
                     return itemConfigure( interp, objc, objv, para, items );
                  case ItemCommandIdx:
                     return itemCommand( interp, objc, objv, para, items );
                  case AffineIdx:
                     return affine( interp, objc, objv, para, items, Affine );
                  case ScaleIdx:
                     return affine( interp, objc, objv, para, items, Scale );
                  case MoveIdx:
                     return affine( interp, objc, objv, para, items, Move );
                  case RotateIdx:
                     return affine( interp, objc, objv, para, items, Rotate );
                  case GetBoundsIdx:
                     return itemBounds( interp, objc, objv, para, items );
               }
            }
      case WindowToWorldIdx:
            return windowToWorld( interp, objc, objv, para, 0 );
      case WorldToWindowIdx:
            return windowToWorld( interp, objc, objv, para, 1 );
      case FindItemAtIdx:
            return findItemAt( interp, objc, objv, para );
   }

   return TCL_OK;
}

int gnoclCanvasCmd( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{
   CanvasParams *para;
   int          ret;

   if( gnoclParseOptions( interp, objc, objv, canvasOptions ) != TCL_OK )
   {
      gnoclClearOptions( canvasOptions );
      return TCL_ERROR;
   }

   para = g_new( CanvasParams, 1 );

   /* what is that for? Found in canvas demos. */
   gtk_widget_push_colormap( gdk_rgb_get_cmap() );

   /* antialiased is default */
   if( canvasOptions[antialiasedIdx].status == GNOCL_STATUS_CHANGED &&
         canvasOptions[antialiasedIdx].val.b == 0 )
   {
      para->canvas = GNOME_CANVAS( gnome_canvas_new( ) );
   }
   else
      para->canvas = GNOME_CANVAS( gnome_canvas_new_aa( ) );

   gtk_widget_show( GTK_WIDGET( para->canvas ) );

   gtk_widget_pop_colormap(); /* what is that for? Found in canvas demos. */

   gnome_canvas_set_center_scroll_region( para->canvas, 0 );
   ret = gnoclSetOptions( interp, canvasOptions, 
         G_OBJECT( para->canvas ), -1 );
   if( ret == TCL_OK )
      ret = configure( interp, para, canvasOptions );
   gnoclClearOptions( canvasOptions );

   if( ret != TCL_OK )
   {
      gtk_widget_destroy( GTK_WIDGET( para->canvas ) );
      g_free( para );
      return TCL_ERROR;
   }

   para->name = gnoclGetAutoWidgetId();
   gnoclMemNameAndWidget( para->name, GTK_WIDGET( para->canvas ) );

   /* TODO: g_hash_table_new_full */
   para->tags = g_hash_table_new(  g_str_hash, g_str_equal );
   para->interp = interp;
   g_signal_connect_after( G_OBJECT( para->canvas ), "destroy", 
         G_CALLBACK( destroyFunc ), para );

   Tcl_CreateObjCommand( interp, para->name, canvasFunc, para, NULL );

   Tcl_SetObjResult( interp, Tcl_NewStringObj( para->name, -1 ) );

   return TCL_OK;
}

