/*
 * $Id: canvasBPath.c,v 1.2 2003/09/01 18:13:08 baum Exp $
 *
 * This file implements the line item of the canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2002-11: Begin of developement
 */

#include "canvas.h"
#include <string.h>
#include <assert.h>

static GnoclOption bPathOptions[] = 
{
   { "-coords", GNOCL_LIST, NULL },
   { "-tags", GNOCL_LIST, NULL },
   { "-fill", GNOCL_OBJ, "fill_color_gdk", gnoclOptGdkColor },
   { "-outline", GNOCL_OBJ, "outline_color_gdk", gnoclOptGdkColor },
   { "-width", GNOCL_DOUBLE, "width_units" },
   { "-pixelWidth", GNOCL_INT, "width_pixels" },
   { "-capStyle", GNOCL_OBJ, "cap_style", gnoclOptCapStyle },
   { "-joinStyle", GNOCL_OBJ, "join_style", gnoclOptJoinStyle },
   /* what's that for? 
   { "-wind", GNOCL_INT, "wind" },
   { "-miterLimit", GNOCL_DOUBLE, "miterlimit" },
   */
   /* TODO: 
   { "-path", GNOCL_LIST, NULL }, {x y} {x y x y x y} {moveTo lineTo curveTo}
   { "-dash", GNOCL_OBJ, "dash" }, 
   { "-fillStipple", GNOCL_OBJ, "fill_stipple" }, 
   */
   { "-onMotion", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onEnter", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onLeave", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonPress", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonRelease", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { NULL }
};

static const int coordsIdx = 0;
static const int tagsIdx   = 1;

static int setOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info )
{
   gnoclResetSetOptions( bPathOptions );

   if( gnoclSetOptions( interp, bPathOptions, G_OBJECT( info->item ), -1 ) 
         != TCL_OK )
      return TCL_ERROR;

   if( gnoclCanvasHandleTags( interp, bPathOptions + tagsIdx, info ) != TCL_OK )
      return TCL_ERROR;

   if( bPathOptions[coordsIdx].status == GNOCL_STATUS_CHANGED )
   {
      Tcl_Obj *obj = bPathOptions[coordsIdx].val.obj;
      GnomeCanvasPathDef *path;
      int   noCoords;
      int   k;
      double coords[6];

      if( Tcl_ListObjLength( interp, obj, &noCoords ) != TCL_OK
            || noCoords < 8 || ( (noCoords - 2) % 6 != 0 ) )
      {
         Tcl_SetResult( interp, "list-of-coordinates must contain a "
               "multiple of 6 plus two elements (8, 14, 20 ...)", TCL_STATIC );
         return TCL_ERROR;
      }

      for( k = 0; k < 2; ++k )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, obj, k, &tp ) != TCL_OK 
               || Tcl_GetDoubleFromObj( interp, tp, &coords[k] ) != TCL_OK )
         {
            return TCL_ERROR;
         }
      }

      path = gnome_canvas_path_def_new();
      gnome_canvas_path_def_moveto( path, coords[0], coords[1] );

      for( k = 2; k < noCoords; k += 6 )
      {
         int n;
         for( n = 0; n < 6; ++n )
         {
            Tcl_Obj *tp;
            if( Tcl_ListObjIndex( interp, obj, k + n, &tp ) != TCL_OK 
                  || Tcl_GetDoubleFromObj( interp, tp, &coords[n] ) != TCL_OK )
            {
               gnome_canvas_path_def_unref( path );
               return TCL_ERROR;
            }
         }
         gnome_canvas_path_def_curveto( path, coords[0], coords[1], 
               coords[2], coords[3], coords[4], coords[5] );

      }
      assert( k == noCoords );

      g_object_set( G_OBJECT( info->item ), "bpath", path, NULL);
      gnome_canvas_path_def_unref( path );
   }

   return TCL_OK;
}

static int parseOptions( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], Gnocl_CanvasItemInfo *info )
{
   gnoclClearOptions( bPathOptions );
   return gnoclParseOptions( interp, objc, objv, bPathOptions );
}

static int clearOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info ) 
{
   gnoclClearOptions( bPathOptions );
   return TCL_OK;
}

Gnocl_CanvasItemInfo *gnoclCanvasCreateBPath( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   Gnocl_CanvasItemInfo *info = g_new( Gnocl_CanvasItemInfo, 1 );
   info->parseOptions = &parseOptions;
   info->setOptions = &setOptions;
   info->clearOptions = &clearOptions;
   info->command = NULL;
   info->item = gnome_canvas_item_new( group, 
         gnome_canvas_bpath_get_type( ), 
         "fill_color", "black", "outline_color", "black", NULL );

   /* init options */
   gnoclParseOptions( NULL, 0, NULL, bPathOptions );

   return info;
}

