/*
 * $Id: canvasRE.c,v 1.2 2003/09/01 18:13:08 baum Exp $
 *
 * This file implements the rectangle and ellipse item of the canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2002-11: switched from GnoclWidgetOptions to GnoclOption
   2001-05: added rectangle and renamed file canvasEllipse.c to canvasRE.c 
   2001-04: Begin of developement
 */

#include "canvas.h"
#include <string.h>
#include <assert.h>


static int optGdkColorOrNull( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret );

static GnoclOption reOptions[] = 
{
   { "-coords", GNOCL_LIST, NULL },
   { "-centerRadius", GNOCL_BOOL, NULL },
   { "-tags", GNOCL_LIST, NULL },
   { "-fill", GNOCL_OBJ, "fill_color_gdk", optGdkColorOrNull },
   { "-outline", GNOCL_OBJ, "outline_color_gdk", gnoclOptGdkColor },
   { "-width", GNOCL_DOUBLE, "width_units" },
   { "-onMotion", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onEnter", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onLeave", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonPress", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonRelease", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { NULL }
};

static const int coordsIdx       = 0;
static const int centerRadiusIdx = 1;
static const int tagsIdx         = 2;

static int optGdkColorOrNull( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret )
{
   int len;
   Tcl_GetStringFromObj( opt->val.obj, &len );
   if( len == 0 )
   {
      g_object_set( obj, opt->propName, NULL, NULL );
      return TCL_OK;
   }

   return gnoclOptGdkColor( interp, opt, obj, ret );
}

static int setOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info )
{
   gnoclResetSetOptions( reOptions );

   if( gnoclSetOptions( interp, reOptions, G_OBJECT( info->item ), -1 ) 
         != TCL_OK )
      return TCL_ERROR;

   if( gnoclCanvasHandleTags( interp, reOptions + tagsIdx, info ) != TCL_OK )
      return TCL_ERROR;

   if( reOptions[coordsIdx].status == GNOCL_STATUS_CHANGED )
   {
      Tcl_Obj *coordsObj = reOptions[coordsIdx].val.obj;
      int centerRadius = 
            GTK_CHECK_TYPE( info->item, GNOME_TYPE_CANVAS_RECT ) ? 0 : 1;
      double coords[4];
      int   noCoords;
      int   n;
      
      if( reOptions[centerRadiusIdx].status == GNOCL_STATUS_CHANGED )
         centerRadius = reOptions[centerRadiusIdx].val.b;

      if( Tcl_ListObjLength( interp, coordsObj, &noCoords ) != TCL_OK
         || !( noCoords == 4 || ( centerRadius && noCoords == 3 ) ) )
      {
         Tcl_SetResult( interp, 
               "list-of-coordinates must contain "
               "3 (if centerRadius) or 4 elements", TCL_STATIC );
         return TCL_ERROR;
      }

      for( n = 0; n < noCoords; ++n )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, coordsObj, n, &tp ) != TCL_OK 
               || Tcl_GetDoubleFromObj( interp, tp, &coords[n] ) != TCL_OK )
         {
            return TCL_ERROR;
         }
      }
      if( centerRadius )
      {
         double x = coords[0];
         double y = coords[1];
         double rx = coords[2];
         double ry = (noCoords == 3) ? rx : coords[3];
         coords[0] = x - rx;  /* FIXME: offset .5 ? */
         coords[1] = y - ry;
         coords[2] = x + rx;
         coords[3] = y + ry;
      }
      gnome_canvas_item_set( info->item, "x1", coords[0], "y1", coords[1],
            "x2", coords[2], "y2", coords[3], NULL );
   }

   return TCL_OK;
}

static int parseOptions( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], Gnocl_CanvasItemInfo *info )
{
   gnoclClearOptions( reOptions );
   return gnoclParseOptions( interp, objc, objv, reOptions );
}

static int clearOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info ) 
{
   gnoclClearOptions( reOptions );
   return TCL_OK;
}

static Gnocl_CanvasItemInfo *createRE( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group, 
      int isRectangle )
{
   Gnocl_CanvasItemInfo *info = g_new( Gnocl_CanvasItemInfo, 1 );
   info->parseOptions = &parseOptions;
   info->setOptions = &setOptions;
   info->clearOptions = &clearOptions;
   info->command = NULL;
   info->item = gnome_canvas_item_new( group, 
         isRectangle ? gnome_canvas_rect_get_type() : 
               gnome_canvas_ellipse_get_type(),
         "fill_color", "black", NULL );

   /* init options */
   gnoclParseOptions( NULL, 0, NULL, reOptions );

   return info;
}

Gnocl_CanvasItemInfo *gnoclCanvasCreateEllipse( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   return createRE( interp, objc, objv, group, 0 );
}

Gnocl_CanvasItemInfo *gnoclCanvasCreateRectangle( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   return createRE( interp, objc, objv, group, 1 );
}
