/*
 * $Id: canvasRichText.c,v 1.2 2003/09/01 18:13:08 baum Exp $
 *
 * This file implements the rich text item of the canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2002-11: Begin of developement
 */

#include "canvas.h"
#include <string.h>
#include <assert.h>

static GnoclOption textOptions[] = 
{
   { "-coords", GNOCL_LIST, NULL },
   { "-tags", GNOCL_OBJ, NULL },
   { "-text", GNOCL_STRING, "text" },
   { "-width", GNOCL_DOUBLE, "width" },
   { "-height", GNOCL_DOUBLE, "height" },
   { "-editable", GNOCL_BOOL, "editable" },
   { "-visible", GNOCL_BOOL, "visible" },
   { "-cursorVisible", GNOCL_BOOL, "cursor_visible" },
   { "-cursorBlink", GNOCL_BOOL, "cursor_blink" },
   { "-growHeight", GNOCL_BOOL, "grow_height" },
   { "-wrapMode", GNOCL_OBJ, "wrap_mode", gnoclOptWrapmode },
   { "-justify", GNOCL_OBJ, "justification", gnoclOptJustification },
   /* { "-direction", GNOCL_OBJ, "direction", ??? }, */
   { "-anchor", GNOCL_OBJ, "anchor", gnoclOptAnchor }, 
   { "-pixelsAboveLines", GNOCL_INT, "pixels_above_lines" },
   { "-pixelsBelowLines", GNOCL_INT, "pixels_below_lines" },
   { "-pixelsInsideWrap", GNOCL_INT, "pixels_inside_wrap" },
   { "-leftMargin", GNOCL_INT, "left_margin" },
   { "-rightMargin", GNOCL_INT, "right_margin" },
   { "-indent", GNOCL_INT, "indent" },
   { "-onMotion", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onEnter", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onLeave", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonPress", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonRelease", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { NULL }
};

static const int coordsIdx = 0;
static const int tagsIdx   = 1;

static int setOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info )
{
   gnoclResetSetOptions( textOptions );

   if( gnoclSetOptions( interp, textOptions, G_OBJECT( info->item ), -1 ) 
         != TCL_OK )
      return TCL_ERROR;

   if( gnoclCanvasSet2Double( interp, info->item, &textOptions[coordsIdx], 
         "x", "y" ) != TCL_OK )
      return TCL_ERROR;

   if( gnoclCanvasHandleTags( interp, textOptions + tagsIdx, info ) != TCL_OK )
      return TCL_ERROR;

   return TCL_OK;
}

static int parseOptions( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], Gnocl_CanvasItemInfo *info )
{
   gnoclClearOptions( textOptions );
   return gnoclParseOptions( interp, objc, objv, textOptions );
}

static int clearOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info ) 
{
   gnoclClearOptions( textOptions );
   return TCL_OK;
}

static int command( Tcl_Interp *interp, int objc, Tcl_Obj * const objv[], 
         Gnocl_CanvasItemInfo *info )
{
   GtkTextBuffer *buffer = gnome_canvas_rich_text_get_buffer(
         GNOME_CANVAS_RICH_TEXT( info->item ) );

   /* canvas itemCommand tag cmd ?-option value? */
   int ret = gnoclTextCommand( buffer, interp, objc, objv, 3, 0 );
   return ret == 0 ? TCL_OK : TCL_ERROR;
}

#if 0
/* FIXME: this does not work, why?  */
static void setFocus( GObject *item, GdkEventButton *event,
      gpointer data )
{
   if( event->type == GDK_BUTTON_PRESS )
   {
      printf( "in setFocus\n" );
      g_object_set( G_OBJECT( GNOME_CANVAS_ITEM( item )->canvas ), 
            "has-focus", (gboolean) 1, NULL );
      /*
      gnome_canvas_item_grab_focus( GNOME_CANVAS_ITEM( item ) ); 
      */
   }
}
#endif

Gnocl_CanvasItemInfo *gnoclCanvasCreateRichText( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   Gnocl_CanvasItemInfo *info = g_new( Gnocl_CanvasItemInfo, 1 );
   info->parseOptions = &parseOptions;
   info->setOptions = &setOptions;
   info->clearOptions = &clearOptions;
   info->command = &command;
   info->item = gnome_canvas_item_new( group, 
         gnome_canvas_rich_text_get_type( ), 
         "width", 100.0, "height", 100.0, NULL );

   /* FIXME: this does not work, why?
   g_signal_connect( G_OBJECT( info->item ), "event",
         G_CALLBACK( setFocus ), NULL );
   */

   /* init options */
   gnoclParseOptions( NULL, 0, NULL, textOptions );
   return info;
}

