/*
 * $Id: gnoclCanvas.c,v 1.4 2003/09/28 17:28:48 baum Exp $
 *
 * This file implements a Tcl interface to the Gnome canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2003-07:    Split from gnocl as separate library
 */

#include "canvas.h"
#include <string.h>
#include <ctype.h>

#include <assert.h>

Tcl_ObjCmdProc gnoclCanvasCmd;

int gnoclOptJoinStyle( Tcl_Interp *interp, GnoclOption *opt,
      GObject *obj, Tcl_Obj **ret )
{
   const char *txt[] = { "miter", "round", "bevel", NULL };
   const int types[] = { GDK_JOIN_MITER, GDK_JOIN_ROUND, GDK_JOIN_BEVEL };

   assert( sizeof( GDK_JOIN_MITER ) == sizeof( int ) );

   return gnoclOptGeneric( interp, opt, obj, "join style", txt, types, ret );
}

int gnoclOptCapStyle( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret )
{
   const char *txt[] = { "notLast", "butt", "round", "projecting", NULL };
   const int types[] = { GDK_CAP_NOT_LAST, GDK_CAP_BUTT,
          GDK_CAP_ROUND, GDK_CAP_PROJECTING };

   assert( sizeof( GDK_CAP_NOT_LAST ) == sizeof( int ) );

   return gnoclOptGeneric( interp, opt, obj, "cap style", txt, types, ret );
}

int gnoclCanvasSet2Double( Tcl_Interp *interp, GnomeCanvasItem *item,
      GnoclOption *option, const char *str1, const char *str2 )
{
   if( option->status == GNOCL_STATUS_CHANGED )
   {
      double coords[2];
      int    k;

      if( Tcl_ListObjLength( interp, option->val.obj, &k ) != TCL_OK || k != 2 )
      {
         Tcl_SetResult( interp, "list-of-coordinates must contain exactly "
               "2 elements", TCL_STATIC );
         return TCL_ERROR;
      }
      for( k = 0; k < 2; ++k )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, option->val.obj, k, &tp ) != TCL_OK 
               || Tcl_GetDoubleFromObj( interp, tp, &coords[k] ) != TCL_OK )
         {
            return TCL_ERROR;
         }
      }

      gnome_canvas_item_set( item, str1, coords[0], str2, coords[1], NULL );
   }
   return TCL_OK;
}

int Gnoclcanvas_Init( Tcl_Interp *interp )
{

   /* printf( "Initializing gnocl canvas version %s\n", VERSION ); */

   if( Tcl_InitStubs( interp, "8.3", 0 ) == NULL )
      return TCL_ERROR;

   if( Tcl_PkgRequire( interp, "Gnocl", VERSION, 0 ) == NULL )
      return TCL_ERROR;

   if( Tcl_PkgProvide( interp, "GnoclCanvas", VERSION ) != TCL_OK )
      return TCL_ERROR;

   Tcl_CreateObjCommand( interp, "gnocl::canvas", gnoclCanvasCmd, NULL, NULL );
   
   return TCL_OK;
}

